/*****************************************************************************/
#include "tex.h"

D_SetupImage16(int w, int h)
{
   int bytesperpixel=0;
   int rv,gv,bv,i,j;
   double b;
   
   if (depth==8) bytesperpixel=1;
   if (depth==15) bytesperpixel=2;
   if (depth==16) bytesperpixel=2;
   if (depth==24) bytesperpixel=3;
   if (depth==32) bytesperpixel=4;
   image=(char *)xim->data;
   for (i=0;i<256;i++)
     {
	b=(double)i/100;
	for (j=0;j<32;j++)
	  {
	     rv=(int)((double)j*b)<<11;
	     if (rv>0xf800) rv=0xf800;
	     brighttab_r[i][j]=rv;
	  }
	for (j=0;j<64;j++)
	  {
	     gv=(int)((double)j*b)<<5;
	     if (gv>0x07e0) gv=0x07e0;
	     brighttab_g[i][j]=gv;
	  }
	for (j=0;j<32;j++)
	  {
	     bv=(int)((double)j*b);
	     if (bv>0x001f) bv=0x001f;
	     brighttab_b[i][j]=bv;
	  }
     }
}

D_DrawPixel16(int x, int y, unsigned char r, unsigned char g, unsigned char b)
{
  unsigned short *ptr;
  if ((x>=0) && (x<winwidth) && (y>=0) &&(y<winheight))
    {
      ptr=(unsigned short *)image;
      ptr+=y*winwidth+x;
      r=r>>3;
      g=g>>2;
      b=b>>3;
      *ptr=((unsigned short)r<<11)|((unsigned short)g<<5)|(unsigned short)b;
    }
}

int D_AddImage(int sx,int sy,int w,int h,texture *ltex)
{
   int x,y;
   unsigned short *ptr1,*ptr2,*ptr3,data1,data2;
   int x1,y1,x2,y2;
   int iox,ioy;
   int dx,dy;
   int xx,yy;
   int v,vv,vvv;
   
   if (!((sx>=0)&&(sx<winwidth)&&(sy>=0)&&(sy<winheight))) return 1;
   ptr1=(short *)image;
   x1=sx-(w>>1);
   y1=sy-(h>>1);
   x2=sx+(w>>1);
   y2=sy+(h>>1);
   iox=0;
   ioy=0;
   if (x1<0)
     {
	iox=(((-x1)*ltex->width*256)/w);
	x1=0;
     }
   if (y1<0)
     {
	ioy=(((-y1)*ltex->height*256)/h);
	y1=0;
     }
   if (x2>=winwidth) x2=winwidth-1;
   if (y2>=winheight) y2=winheight-1;
   ptr1+=(winwidth*y1)+x1;
   dx=((ltex->width-1)<<8)/w;
   dy=((ltex->height-1)<<8)/h;
   xx=iox;
   yy=ioy;
   for(y=y1;y<=y2;y++)
     {
	ptr2=ptr1;
	for(x=x1;x<=x2;x++)
	  {
	     data1=*ptr2;
	     ptr3=ltex->data;
	     ptr3+=(xx>>8)+(ltex->width*(yy>>8));
	     xx+=dx;
	     data2=*ptr3;
	     v=(int)(data1&0x001f)+(int)(data2&0x001f);
	     if (v>0x001f) v=0x001f;
	     vvv=v;
	     v=(int)(data1&0x07e0)+(int)(data2&0x07e0);
	     if (v>0x07e0) v=0x07e0;
	     vvv+=v;
	     v=(int)(data1&0xf800)+(int)(data2&0xf800);
	     if (v>0xf800) v=0xf800;
	     vvv+=v;
	     *ptr2++=(short)vvv;
	  }
	ptr1+=winwidth;
	xx=iox;
	yy+=dy;
     }
   return 0;
}

D_DrawLight(light *l,texture *ltex, texture *lref1, texture *lref2)
{
   int sx,sy,w,h,d;
   int ssx,ssy;
   if (l->z<-focus) return 0;
   ssx=(int)((l->x*focus)/(l->z+focus));
   ssy=(int)((l->y*focus)/(l->z+focus));
   d=(int)(256*focus)/(l->z+focus);
   if (d<=0) return 0;
   sx=(winwidth>>1)+ssx;
   sy=(winheight>>1)+ssy;
   w=(d*ltex->width)>>8;
   h=(d*ltex->height)>>8;
   if(D_AddImage(sx,sy,w,h,ltex)) return 0;
   sx=(winwidth>>1)-ssx;
   sy=(winheight>>1)-ssy;
   w=lref1->width;
   h=lref1->height;
   D_AddImage(sx,sy,w,h,lref1);
   sx=(winwidth>>1)-((ssx*200)>>8);
   sy=(winheight>>1)-((ssy*200)>>8);
   w=lref2->width;
   h=lref2->height;
   D_AddImage(sx,sy,w,h,lref2);
   sx=(winwidth>>1)-((ssx*160)>>8);
   sy=(winheight>>1)-((ssy*160)>>8);
   w=lref1->width>>2;
   h=lref1->height>>2;
   D_AddImage(sx,sy,w,h,lref1);
}


texture *D_LoadTexture16(char *file)
{
   FILE *f;
   texture *tex;
   unsigned char r,g,b,s[256];
   int done,width,height,x,y;
   unsigned short *data;
   short **lineptr;
   tex=(texture *)malloc(sizeof(texture));
   f=fopen(file,"r");
   if (!f) 
     {
	fprintf(stderr,"ERROR: Cannot load image file:\n%s\nEXIT...\n",file);
	exit(1);
     }
   fgets(s,256,f);
   s[strlen(s)-1]=0; 
   if (strcmp(s,"P6"))
     { 
	fprintf(stderr,"ERROR: Cannot load image file:\n%s\nEXIT...\n",file);
	exit(1);
     }
   done=1;
   while (done)
     {
	fgets(s,256,f);
	s[strlen(s)-1]=0;
	if (s[0]!='#')
	  {
	     done=0;
	     sscanf(s,"%i %i",&width,&height);
	     tex->width=width;
	     tex->height=height;
	     data=(short *)malloc(width*height*sizeof(short));
	     tex->data=data;
	     lineptr=(short **)malloc(height*sizeof(short *));
	     tex->lineptr=lineptr;
	     fgets(s,256,f);
	     s[strlen(s)-1]=0;
	     for (y=0;y<height;y++)
	       {
		  lineptr[y]=data;
		  for (x=0;x<width;x++)
		    {
		       fread(&r,1,1,f);
		       fread(&g,1,1,f);
		       fread(&b,1,1,f);
		       r=r>>3;
		       g=g>>2;
		       b=b>>3;
		       *data++=((short)r<<11)|
			 ((short)g<<5)|
			 (short)b;
		    }
	       }
	  }
     } 
   fclose(f);
   return tex;
}

D_DrawTexture16(int sx, int sy, texture *tex)
{
  int x,y,mx,my;
  unsigned short *src,*dst;
  
  if ((sx<winwidth) && (sy<winheight))
    {
      mx=tex->width+sx;
      my=tex->height+sy;
      if (mx>winwidth) mx=winwidth;
      if (my>winheight) my=winheight;
      for (y=sy;y<my;y++)
	{
	  dst=(unsigned short*)image+y*winwidth+sx;
	  src=tex->data+((y-sy)*tex->width);
	  for (x=sx;x<mx;x++)
	    {
	      *dst++=*src++;
	    }
	}
    }
}

D_ClearImage16()
{
   int i,size;
   int *ptr;
   unsigned int r,g,b,val;
   
   r=15;g=31;b=15;
   val=0;
   val=(r<<11)|(g<<5)|b;
   val=val+(val<<16);
   size=winwidth*winheight/2;
   ptr=(int *)image;
   for(i=0;i<size;i++)
     {
	*ptr++=val;
     }
}

D_Draw3DTriangle16(triangle *t,light *l)
{
   short *tex,*src,*img;
   short **tline;
   int sx1,sy1,sx2,sy2,sx3,sy3;
   int lsx,lsy;
   int t1,t2;
   int x,y;
   int left,right;
   int p1u,p1v,p2u,p2v,p3u,p3v;
   int texu,texv;
   int texb;
   int val,rv,gv,bv;
   int twidth,theight;
   int ddu,ddv,ddb;
   int uu,vv,bb;
   int ret=0;
   double u,v,b;
   double p1b,p2b,p3b;
   double u1,v1,u2,v2,u3,v3;
   double b1,b2,b3;
   double dul,dvl,dur,dvr;
   double dbl,dbr;
   double du,dv;
   double db;
   double tfp;
   double uleft,vleft;
   double uright,vright;
   double bleft,bright;
   double len,lp,rp;
   double ldx,rdx;
   double uul,vvl,uur,vvr;
   double bbl,bbr;
   double avz;
   
   sx1=(winwidth>>1)+(int)((t->p1->x*focus)/(t->p1->z+focus));
   sy1=(winheight>>1)+(int)((t->p1->y*focus)/(t->p1->z+focus));
   sx2=(winwidth>>1)+(int)((t->p2->x*focus)/(t->p2->z+focus));
   sy2=(winheight>>1)+(int)((t->p2->y*focus)/(t->p2->z+focus));
   sx3=(winwidth>>1)+(int)((t->p3->x*focus)/(t->p3->z+focus));
   sy3=(winheight>>1)+(int)((t->p3->y*focus)/(t->p3->z+focus));
   lsx=(winwidth>>1)+(int)((l->x*focus)/(l->z+focus));
   lsy=(winheight>>1)+(int)((l->y*focus)/(l->z+focus));
   avz=(t->p1->z+t->p2->z+t->p3->z)/3;
   
   p1u=t->surf->p1.u;
   p1v=t->surf->p1.v;
   p2u=t->surf->p2.u;
   p2v=t->surf->p2.v;
   p3u=t->surf->p3.u;
   p3v=t->surf->p3.v;
   p1b=t->n1->bright;
   p2b=t->n2->bright;
   p3b=t->n3->bright;

   if (sy2<sy1)
     {
	t1=sx1;t2=sy1;
	sx1=sx2;sy1=sy2;
	sx2=t1;sy2=t2;
	t1=p1u;t2=p1v;
	p1u=p2u;p1v=p2v;
	p2u=t1;p2v=t2;
	tfp=p1b;
	p1b=p2b;
	p2b=tfp;
     }
   if (sy3<sy2)
     {
	t1=sx2;t2=sy2;
	sx2=sx3;sy2=sy3;
	sx3=t1;sy3=t2;
	t1=p2u;t2=p2v;
	p2u=p3u;p2v=p3v;
	p3u=t1;p3v=t2;
	tfp=p2b;
	p2b=p3b;
	p3b=tfp;
     }
   if (sy2<sy1)
     {
	t1=sx1;t2=sy1;
	sx1=sx2;sy1=sy2;
	sx2=t1;sy2=t2;
	t1=p1u;t2=p1v;
	p1u=p2u;p1v=p2v;
	p2u=t1;p2v=t2;
	tfp=p1b;
	p1b=p2b;
	p2b=tfp;
     }
   if (sy1>=winheight) return 0;
   if (sy3<0) return 0;
   if ((sx1<0)&&(sx2<0)&&(sx3<0)) return 0;
   if ((sx1>=winwidth)&&(sx2>=winwidth)&&(sx3>=winwidth)) return 0;
   u1=(double)p1u;
   v1=(double)p1v;
   u2=(double)p2u;
   v2=(double)p2v;
   u3=(double)p3u;
   v3=(double)p3v;
   b1=p1b;
   b2=p2b;
   b3=p3b;
   dul=(u2-u1)/((double)(sy2-sy1));
   dvl=(v2-v1)/((double)(sy2-sy1));
   dur=(u3-u1)/((double)(sy3-sy1));
   dvr=(v3-v1)/((double)(sy3-sy1));
   dbl=(b2-b1)/((double)(sy2-sy1));
   dbr=(b3-b1)/((double)(sy3-sy1));
   uul=u1;
   vvl=v1;
   uur=u1;
   vvr=v1;
   bbl=b1;
   bbr=b1;
   ldx=(double)(sx2-sx1)/(double)(sy2-sy1);
   rdx=(double)(sx3-sx1)/(double)(sy3-sy1);
   lp=(double)sx1;
   rp=lp;
   tex=t->surf->tex->data;
   twidth=t->surf->tex->width;
   theight=t->surf->tex->height;
   tline=t->surf->tex->lineptr;
   if (sy2>=winheight) sy2=winheight-1;
   for (y=sy1;y<sy2;y++)
     {
	if (lp<rp)
	  {
	     left=(int)lp;
	     right=(int)rp;
	     uleft=uul;
	     uright=uur;
	     vleft=vvl;
	     vright=vvr;
	     bleft=bbl;
	     bright=bbr;
	  }
	else
	  {
	     left=(int)rp;
	     right=(int)lp;
	     uleft=uur;
	     uright=uul;
	     vleft=vvr;
	     vright=vvl;
	     bleft=bbr;
	     bright=bbl;
	  }
	if ((lsy==y)&&(left<=lsx)&&(lsx<=right)&&(l->z>avz)) ret=1;
	if (y>=0)
	  {
	     len=(double)(1+right-left);
	     du=(uright-uleft)/len;
	     dv=(vright-vleft)/len;
	     db=(bright-bleft)/len;
	     u=uleft;
	     v=vleft;
	     b=bleft;
	     if (right>=0)
	       {
		  while (++left<0)
		    {
		       u+=du;
		       v+=dv;
		       b+=db;
		    }
		  if (right>=winwidth) right=winwidth-1;
		  img=(short *)image+(y*winwidth)+left;
		  if ((bleft==0)&&(bright==0))
		    {
		       for (x=left;x<=right;x++)
			 {
			    *img++=0;
			 }
		    }
		  else
		    {
		       ddu=(int)(du*256);
		       ddv=(int)(dv*256);
		       ddb=(int)(db*256);
		       uu=(int)(u*256);
		       vv=(int)(v*256);
		       bb=(int)(b*256);
		       for (x=left;x<=right;x++)
			 {
			    texu=uu>>8;
			    texv=vv>>8;
			    texb=bb>>8;
			    uu+=ddu;
			    vv+=ddv;
			    bb+=ddb;
			    src=(short *)tline[texv]+texu;
			    val=*src;
			    rv=brighttab_r[texb][(val>>11)&0x1f];
			    gv=brighttab_g[texb][(val>>5)&0x3f];
			    bv=brighttab_b[texb][val&0x1f];
			    *img++=rv|gv|bv;
			 }
		    }
	       }
	  }
	lp+=ldx;
	rp+=rdx;
	uul+=dul;
	vvl+=dvl;
	uur+=dur;
	vvr+=dvr;
	bbl+=dbl;
	bbr+=dbr;
     }
   if (sy2>=winheight) return 0;
   dul=(u3-u2)/((double)(sy3-sy2));
   dvl=(v3-v2)/((double)(sy3-sy2));
   dbl=(b3-b2)/((double)(sy3-sy2));
   uul=u2;
   vvl=v2;
   bbl=b2;
   ldx=(double)(sx3-sx2)/(double)(sy3-sy2);
   lp=(double)sx2;
   if (sy3>=winheight) sy3=winheight-1;
   for (y=sy2;y<=sy3;y++)
     {
	if (lp<rp)
	  {
	     left=(int)lp;
	     right=(int)rp;
	     uleft=uul;
	     uright=uur;
	     vleft=vvl;
	     vright=vvr;
	     bleft=bbl;
	     bright=bbr;
	  }
	else
	  {
	     left=(int)rp;
	     right=(int)lp;
	     uleft=uur;
	     uright=uul;
	     vleft=vvr;
	     vright=vvl;
	     bleft=bbr;
	     bright=bbl;
	  }
	if ((lsy==y)&&(left<=lsx)&&(lsx<=right)&&(l->z>avz)) ret=1;
	if (y>=0)
	  {
	     len=(double)(1+right-left);
	     du=(uright-uleft)/len;
	     dv=(vright-vleft)/len;
	     db=(bright-bleft)/len;
	     u=uleft;
	     v=vleft;
	     b=bleft;
	     if (right>=0)
	       {
		  while (++left<0)
		    {
		       u+=du;
		       v+=dv;
		       b+=db;
		    }
		  if (right>=winwidth) right=winwidth-1;
		  img=(short *)image+(y*winwidth)+left;
		  if ((bleft==0)&&(bright==0))
		    {
		       for (x=left;x<=right;x++)
			 {
			    *img++=0;
			 }
		    }
		  else
		    {
		       ddu=(int)(du*256);
		       ddv=(int)(dv*256);
		       ddb=(int)(db*256);
		       uu=(int)(u*256);
		       vv=(int)(v*256);
		       bb=(int)(b*256);
		       for (x=left;x<=right;x++)
			 {
			    texu=uu>>8;
			    texv=vv>>8;
			    texb=bb>>8;
			    uu+=ddu;
			    vv+=ddv;
			    bb+=ddb;
			    src=(short *)tline[texv]+texu;
			    val=*src;
			    rv=brighttab_r[texb][(val>>11)&0x1f];
			    gv=brighttab_g[texb][(val>>5)&0x3f];
			    bv=brighttab_b[texb][val&0x1f];
			    *img++=rv|gv|bv;
			 }
		    }
	       }
	  }
	lp+=ldx;
	rp+=rdx;
	uul+=dul;
	vvl+=dvl;
	uur+=dur;
	vvr+=dvr;
	bbl+=dbl;
	bbr+=dbr;
     }
   return ret;
}

D_DrawScene(world *wld)
{
   int ldraw=1;
   int i,j;
   
   for (i=wld->tnum-1;i>=0;i--)
     {
	j=wld->trilist[i];
	if (wld->see[j])
	  {
                 if (D_Draw3DTriangle16(&wld->tris[j],wld->lt)) ldraw=0;
	  }
     }
   return ldraw;
}

