/*****************************************************************************/

/* DEFINES */
#define M_PI_2          1.57079632679489661923

/* STRUCTURES */

struct sortlist
{
   int index;
   double value;
};

typedef struct
{
  int width;
  int height;
  short *data;
  short **lineptr;
} texture;

typedef struct
{
  double x;
  double y;
  double z;
} point;

typedef struct
{
  double x;
  double y;
  double z;
  double bright;
} light;

typedef struct
{
  int u;
  int v;
} coord;

typedef struct
{
  double x;
  double y;
  double z;
  double bright;
} normal;

typedef struct
{
  texture *tex;
  coord p1;
  coord p2;
  coord p3;
} surface;

typedef struct
{
  point *p1;
  point *p2;
  point *p3;
  surface *surf;
  normal *n1;
  normal *n2;
  normal *n3;
  normal *n;
} triangle;

typedef struct
{
  double ax;
  double ay;
  double az;
} angle;

typedef struct
{
   int pnum;
   int tnum;
   int nnum;
   int cnum;
   point **plist;
   triangle **tlist;
   normal **nlist;
   void **child;
   point loc;
   point wloc;
   angle rot;
   angle wrot;
} group;

typedef struct
{
   int pnum;
   int tnum;
   int nnum;
   int inum;
   int onum;
   int snum;
   
   light *lt;

   texture *lglow;
   texture *lref1;
   texture *lref2;
   
   point *pnts;
   triangle *tris;
   group *objs;
   normal *norms;
   surface *surfs;
   texture **imgs;
   
   int *see;
   int *trilist;
} world;

/* GLOBAL VARIABLES */

double focus;

world *M_LoadWorld(char *file);

int M_InitObj(group *o);

int M_InitChild(group *o, group *c);

int M_RotateObj(group *o, double ax, double ay, double az);

int M_RotateChild(group *o, group *c, double ax, double ay, double az);

int M_CalcTriLighting(triangle *t, int num, light *l, double focus,int *see);

int M_TranslateObj(group *o, point *p);

int M_CalcObjNormals(group *o);

int M_RotateNormal(normal *p,double ax, double ay, double az);

int M_SortTriangles(triangle *tris, int *list, int size);

int M_RotatePoint(point *p, double ax, double ay, double az);

int M_RotateLight(light *p,double ax, double ay, double az);

int M_CalcNormal(point *p1, point *p2, point *p3, normal *n);

int M_CalcLighting(triangle *t, light *l, double focus);

int M_AddPoint(point *p, point *offset);

int M_SubPoint(point *p, point *offset);

int M_AddLight(light *l, point *offset);

int M_SubLight(light *l, point *offset);








































