/*****************************************************************************/
#include "tex.h"

main(int argc, char **argv)
{  
   int x,y,w,h;
   
   if (argc<5) 
     {
	printf("Usage: tex x y width height file.wld\n");
	exit(1);
     }
   x=atoi(argv[1]);
   y=atoi(argv[2]);
   w=atoi(argv[3]);
   h=atoi(argv[4]);
   X_OpenWindow(x,y,w,h);
   if (depth==16)
     {
	X_SetupImage(w,h);
	D_SetupImage16(w,h);
     }
   else 
     {
	fprintf(stderr,"Sorry. Only 16 bit displays are supported\n");
	exit(1);
     }
   strcpy(worldfile,argv[5]);
   run();
}

run()
{
   /* General data structs needed */
   double ax,ay,az;
   double lax,lay,laz;
   int i;
   int x,y;
   texture **tex;
   texture *lglow;
   texture *lref1;
   texture *lref2;
   light *lt;
   triangle *tris;
   normal *norms;
   point *pnts;
   surface *surfs;
   group *obj;
   int *trilist;
   world *wld;
   
   /* data structs and variables needed for timing purposes */
   int frames=0;
   int sec1,usec1;
   int sec2,usec2;
   int times,timeu;
   double time;
   double totaltime;
   struct timeval timev;
   
   wld=M_LoadWorld(worldfile);
   trilist=wld->trilist;
   lt=wld->lt;
   tris=wld->tris;
   pnts=wld->pnts;
   norms=wld->norms;
   surfs=wld->surfs;
   obj=wld->objs;
   tex=wld->imgs;
   lref1=wld->lref1;
   lref2=wld->lref2;
   lglow=wld->lglow;
   
   ax=0.04;
   ay=0.01;
   az=0.001;
   lax=0.002;
   lay=0.004;
   laz=0.001;

   focus=256;
   
   for(i=0;i<wld->onum;i++) 
     {
	M_InitObj(&obj[i]);
	M_CalcObjNormals(&obj[i]);
     }
   M_AddLight(lt,&obj[0].loc);
   
   for (;;)
     {
	gettimeofday(&timev,NULL); /* this secdion just gets the current time */
	sec1=(int)timev.tv_sec; /* and stores it so we can time outselves */
	usec1=(int)timev.tv_usec; /* we will use this to vary speed of rot */
	
	D_ClearImage16(); /* clear the image buffer */
	
	if ((rand()%100)==5) /* 1 in 100 chance of rotation direction change */
	  { /* now pick a nerw dir of rotation randomly */
	     i=rand()%3; /* rotation speed is in rad/sec */
	     if(i==0) ax=(((double)(rand()%10000))/20000)-0.25;
	     if(i==1) ay=(((double)(rand()%10000))/20000)-0.25;
	     if(i==2) az=(((double)(rand()%10000))/20000)-0.25;
	  }
	if ((rand()%250)==5) /* same as above.. but for light */
	  {
	     i=rand()%3;
	     if(i==0) lax=(((double)(rand()%10000))/20000)-0.25;
	     if(i==1) lay=(((double)(rand()%10000))/20000)-0.25;
	     if(i==2) laz=(((double)(rand()%10000))/20000)-0.25;
	  }
	
	for (i=0;i<wld->onum;i++)
	  M_RotateObj(&obj[i],
		      ax*time*(double)(((i/3)%3)-1),
		      ay*time*(double)(((i/7)%3)-1),
		      az*time*(double)((i%3)-1));

	M_SubLight(lt,&obj[0].wloc);
	M_RotateLight(lt,lax*time,lay*time,laz*time);
	M_AddLight(lt,&obj[0].wloc);
	
	M_CalcTriLighting(tris,wld->tnum,lt,focus);
	M_SortTriangles(wld,tris,trilist,wld->tnum);
	
	if (D_DrawScene(wld)) 
	  D_DrawLight(lt,lglow,lref1,lref2);
	
	X_DisplayImage();
	
	gettimeofday(&timev,NULL);
	sec2=(int)timev.tv_sec;
	usec2=(int)timev.tv_usec;
	times=sec2-sec1;
	timeu=usec2-usec1;
	if (timeu<0)
	  {
	     timeu+=1000000;
	     times-=1;
	  }
	time=(double)times+(double)timeu/1000000;
	totaltime+=time;
	frames++;
	if (frames==256)
	  {
	     printf("fps   : %3.1f\n",(double)frames/totaltime);
	     totaltime=0;frames=0;
	  }  
     }
}
